@@ -14,6 +14,10 @@ void pyfastx_sequence_continue_read(pyfastx_Sequence* self) {
 	Py_ssize_t gap;
 	Py_ssize_t rlen;
 
+	if (self->raw) {
+		return;
+	}
+
 	//raw string offset and bytelen
 	header_len = self->desc_len + self->end_len + 1;
 	offset = self->offset - header_len;
@@ -560,25 +564,21 @@ PyObject *pyfastx_sequence_gc_content(pyfastx_Sequence *self, void* closure) {
 	int ret;
 	char *seq;
 
-	sqlite3_stmt *stmt;
-
 	Py_ssize_t i, n;
 	Py_ssize_t a = 0, c = 0, g = 0, t = 0;
-
-	const char *sql = "SELECT * FROM comp WHERE seqid=?";
 	
 	PYFASTX_SQLITE_CALL(
-		sqlite3_prepare_v2(self->index->index_db, sql, -1, &stmt, NULL);
-		sqlite3_bind_int64(stmt, 1, self->id);
-		ret = sqlite3_step(stmt);
+		sqlite3_bind_int64(self->index->comp_stmt, 1, self->id);
+		ret = sqlite3_step(self->index->comp_stmt);
 	);
 
 	if (ret == SQLITE_ROW && self->start == 1 && self->end == self->seq_len) {
 		while (ret == SQLITE_ROW) {
+			
 			PYFASTX_SQLITE_CALL(
-				l = sqlite3_column_int(stmt, 2);
-				n = sqlite3_column_int64(stmt, 3);
-				ret = sqlite3_step(stmt);
+				l = sqlite3_column_int(self->index->comp_stmt, 2);
+				n = sqlite3_column_int64(self->index->comp_stmt, 3);
+				ret = sqlite3_step(self->index->comp_stmt);
 			);
 
 			switch (l) {
@@ -631,7 +631,7 @@ PyObject *pyfastx_sequence_gc_content(pyfastx_Sequence *self, void* closure) {
 		}
 	}
 
-	PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
+	PYFASTX_SQLITE_CALL(sqlite3_reset(self->index->comp_stmt));
 
 	return Py_BuildValue("f", (float)(g+c)/(a+c+g+t)*100);
 }
@@ -640,25 +640,21 @@ PyObject *pyfastx_sequence_gc_skew(pyfastx_Sequence *self, void* closure) {
 	int l;
 	int ret;
 	char *seq;
-	sqlite3_stmt *stmt;
 
 	Py_ssize_t i, n;
 	Py_ssize_t c = 0, g = 0;
-
-	const char *sql = "SELECT * FROM comp WHERE seqid=?";
 	
 	PYFASTX_SQLITE_CALL(
-		sqlite3_prepare_v2(self->index->index_db, sql, -1, &stmt, NULL);
-		sqlite3_bind_int64(stmt, 1, self->id);
-		ret = sqlite3_step(stmt);
+		sqlite3_bind_int64(self->index->comp_stmt, 1, self->id);
+		ret = sqlite3_step(self->index->comp_stmt);
 	);
 
 	if (ret == SQLITE_ROW && self->start == 1 && self->end == self->seq_len) {
 		while (ret == SQLITE_ROW) {
 			PYFASTX_SQLITE_CALL(
-				l = sqlite3_column_int(stmt, 2);
-				n = sqlite3_column_int64(stmt, 3);
-				ret = sqlite3_step(stmt);
+				l = sqlite3_column_int(self->index->comp_stmt, 2);
+				n = sqlite3_column_int64(self->index->comp_stmt, 3);
+				ret = sqlite3_step(self->index->comp_stmt);
 			);
 
 			switch (l) {
@@ -691,7 +687,7 @@ PyObject *pyfastx_sequence_gc_skew(pyfastx_Sequence *self, void* closure) {
 		}
 	}
 
-	PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
+	PYFASTX_SQLITE_CALL(sqlite3_reset(self->index->comp_stmt));
 
 	return Py_BuildValue("f", (float)(g-c)/(g+c));
 }
@@ -701,8 +697,6 @@ PyObject *pyfastx_sequence_composition(pyfastx_Sequence *self, void* closure) {
 	int l;
 	int ret;
 	char *seq;
-
-	sqlite3_stmt *stmt;
 	
 	Py_ssize_t n;
 	Py_ssize_t seq_comp[128] = {0};
@@ -711,12 +705,9 @@ PyObject *pyfastx_sequence_composition(pyfastx_Sequence *self, void* closure) {
 	PyObject *b;
 	PyObject *c;
 
-	const char *sql = "SELECT * FROM comp WHERE ID=?";
-
 	PYFASTX_SQLITE_CALL(
-		sqlite3_prepare_v2(self->index->index_db, sql, -1, &stmt, NULL);
-		sqlite3_bind_int64(stmt, 1, self->id);
-		ret = sqlite3_step(stmt);
+		sqlite3_bind_int64(self->index->comp_stmt, 1, self->id);
+		ret = sqlite3_step(self->index->comp_stmt);
 	);
 
 	d = PyDict_New();
@@ -724,9 +715,9 @@ PyObject *pyfastx_sequence_composition(pyfastx_Sequence *self, void* closure) {
 	if (ret == SQLITE_ROW && self->start == 1 && self->end == self->seq_len) {
 		while (ret == SQLITE_ROW) {
 			PYFASTX_SQLITE_CALL(
-				l = sqlite3_column_int(stmt, 2);
-				n = sqlite3_column_int64(stmt, 3);
-				ret = sqlite3_step(stmt);
+				l = sqlite3_column_int(self->index->comp_stmt, 2);
+				n = sqlite3_column_int64(self->index->comp_stmt, 3);
+				ret = sqlite3_step(self->index->comp_stmt);
 			);
 
 			if (n > 0 && l >= 32 && l < 127) {
@@ -757,8 +748,7 @@ PyObject *pyfastx_sequence_composition(pyfastx_Sequence *self, void* closure) {
 		}
 	}
 
-	PYFASTX_SQLITE_CALL(sqlite3_finalize(stmt));
-
+	PYFASTX_SQLITE_CALL(sqlite3_reset(self->index->comp_stmt));
 	return d;
 }
 
